# Megan Clark
# Data Structures
# File Reader

import random, math, time

class State:
    def __init__(self, name, description=None):
        self.name = name
        self.description = description
        self.edges = []
        self.objects = []
        self.constraints = []

class Edge:
    def __init__(self, edgename, from_node, commands, to_node, health=None, constraints=None):
        self.edgename = edgename
        self.from_node = from_node
        self.commands = commands
        self.to_node = to_node
        self.health = health
        self.constraints = constraints

class Backpack:
    def __init__(self, max_weight=None, value=None):
        self.max_weight = max_weight
        self.value = value
        self.objects = []

class Objects:
    def __init__(self, location=None, name=None, short_description=None, description=None, weight=None):
        self.location = location
        self.name = name
        self.short_description = short_description
        self.description = description
        self.weight = weight

    def __repr__(self):
        return self.name

class Constraint:
    def __init__(self, name, object_name, state_name, weight=None):
        self.name = name
        self.object_name = object_name
        self.state_name = state_name
        self.weight = weight

class Engine:
    def __init__(self, filename):
        self.filename = filename
        self.initial_state = None
        self.initial_health = 10
        self.initial_count = 0
        self.initial_time = 0
        self.backpack_weight = 0
        self.current_state = self.initial_state
        self.current_health = self.initial_health
        self.current_count = self.initial_count
        self.initial_time = time.clock()
        self.backpack = Backpack()


    # Function to load the game
    def load(self):
        self.states = {}
        edges = []
        objects = []
        constraints = []
        fp = open(self.filename)

        # Reading the text file
        for line in fp:
            if len(line) == 0 or line[0] == "#":
                continue
            items = line.strip().split("::")
            if items[0] == "node":
                s = State(*items[1:])
                if self.initial_state is None:
                    self.initial_state = s
                self.states[items[1]] = s
            elif items[0] == "edge":
                edges.append(Edge(*items[1:]))
            elif items[0] == "object":
                obj = Objects(*items[1:])
                objects.append(obj)
            elif items[0] == "constraint":
                constraints.append(Constraint(*items[1:]))
        for edge in edges:
            self.states[edge.from_node].edges.append(edge)
        for o in objects:
            self.states[o.location].objects.append(o)
        for constraint in constraints:
            self.states[constraint.state_name].constraints.append(constraint)


    # Function to reset the game back to the beginning settings
    def reset(self):
        self.current_state = self.initial_state
        self.current_health = self.initial_health
        self.current_count = self.initial_count
        self.initial_time = time.clock()
        self.current_time = 0
        self.backpack = Backpack()


    # Function to get an item
    def get(self, item):
        self.item = item
        self.item.hold = True
        self.item.location = self.backpack
        self.backpack.objects.append(self.item)
        self.current_state.objects.remove(self.item)
        print("The " + self.item.name + " is now in your backpack.")


    # Function to use an item
    def use(self, item):
        self.item = item
        self.item.hold = False
        self.item.location = self.current_state.name
        self.current_state.objects.append(self.item)
        print("You have used the " + self.item.name)


    # Function to drop an item
    def drop(self, location):
        self.item = item
        self.item.hold = False
        self.item.location = self.current_state.name
        self.current_state.objects.append(self.item)
        print("You have dropped the " + self.item.name)


    # Playing the game
    def play(self):
        self.reset()
        objects = Objects()
        while True:
            
            # Game Over if health reaches zero
            if self.current_health <= 0:
                conts = input("Your health has reached zero. Game over. Would you like to play again? ")
                if conts.lower() == "yes":
                    self.reset()
                else:
                    return

            # Game Over if move count reaches 100
            if self.current_count >= 100:
                conts = input("You have made too many moves. Game over. Would you like to play again? ")
                if conts.lower() == "yes":
                    self.reset()
                else:
                    return

            # Game Over if time takes too long
            if self.current_time >= 600:
                conts = input("You have run out of time. Game over. Would you like to play again? ")
                if conts.lower() == "yes":
                    self.reset()
                else:
                    return

            # Print state description upon entering room
            print(self.current_state.description)

            # If user reaches exit, end game
            if str(self.current_state.name) == "exit":
                again = input("Congratulations! You have survived the House. Would you like to play again? ")
                if again[0].lower() == "y":
                    self.reset()
                    print(self.current_state.description)
                else:
                    return

            # If user reaches bottom of stairs, end game
            if str(self.current_state.name) == "bottom":
                again = input("Your energy runs out, and you are left in the House. Game over. Would you like to play again? ")
                if again[0].lower() == "y":
                    self.reset()
                    print(self.current_state.description)
                else:
                    return

            # Print health and move count, as well as command field
            print("Your health is " + str(self.current_health) + " and you have moved " + str(self.current_count) + " times.")
            self.current_time = time.clock() - self.initial_time
            command = input("Command: ").lower().split(" ")

            # If user wants to exit, exit
            if command[0] == "exit":
                return

            # If user wants to give up, give up
            if command[0] == "give":
                if command[1] == "up":
                    print("You have abandoned all hope. The end is nowhere in sight. There is nothing left to do but wait as the growling gets louder.")
                    again = input("Would you like to play again? ").lower().split()
                    if again[0].lower() == "yes":
                        self.reset()
                    else:
                        return

            # Chooses a random room from the list of available to_nodes
            for edge in self.current_state.edges:
                if command[0] in edge.commands:
                    states = edge.to_node.split(", ")
                    randint = math.floor(random.random()*(len(states)-1))
                    if states[randint] in self.states:
                        self.current_state = self.states[states[randint]]

                        # If health is used in the game, increase health accordingly
                        if edge.health != None:
                            self.current_health = self.current_health + int(edge.health)

                        # Increase the counter each move
                        self.current_count = self.current_count + 1

is
            # Handling objects in the room:
            for o in self.current_state.objects:
                
                # Looking at an object/around
                if "look" in command:
                    if o.name in command:
                        print(o.short_description)

                # If object in room, print description
                print(o.description)

                # Getting an object
                if "get" in command:
                    if command[1] in o.name:
                        self.get(o)
                        self.backpack_weight = self.backpack_weight + int(o.weight)

            # Handling objects in the backpack
            for o in self.backpack.objects:

                # Looking at the contents of the backpack
                if command[0] == "backpack":
                    print(self.backpack.objects)

                # Using an object from the backpack (can only use an item from the backpack)
                elif "use" in command:
                    if command[1] in o.name:
                        if self.item.hold == True:
                            self.use(o)
                            self.backpack.objects.remove(o)
                            self.current_health = self.current_health + 1
                        else:
                            print("You aren't holding that item yet.")

                # Dropping an object from the backpack
                elif "drop" in command:
                    if command[1] in o.name:
                        self.drop(o)
                        self.backpack.objects.remove(o)

                # Looking at an object in the backpack
                elif "look" in command:
                    if o.name in command:
                        print(o.short_description)
                
                                         
game = Engine("HouseofLeaves.txt")
game.load()
game.play()
